require 'singleton'
require 'unrest/configuration_hash'

class Unrest
  def self.configure(options = nil, &block)
    if !options.nil?
      Configuration.instance.configure(options)
    end
  end

  def self.config
    Configuration.instance.data
  end

  class Configuration
    include Singleton

    OPTIONS = [
      :https,
      :timeout,
      :headers,
      :parameters,
      :authentication
    ]

    attr_accessor :data

    def self.set_defaults
      instance.set_defaults
    end

    OPTIONS.each do |o|
      define_method o do
        @data[o]
      end
      define_method "#{o}=" do |value|
        @data[o] = value
      end
    end

    def configure(options)
      @data.rmerge!(options)
    end

    def initialize
      @data = Unrest::ConfigurationHash.new
      set_defaults
    end

    def set_defaults
      @data[:https] = false
      @data[:timeout] = 5
      @data[:headers] = {}
      @data[:parameters] = {}
      @data[:authentication] = {}
    end

    instance_eval(OPTIONS.map do |option|
      o = option.to_s
      <<-EOS
      def #{o}
        instance.data[:#{o}]
      end
      def #{o}=(value)
        instance.data[:#{o}] = value
      end
      EOS
    end.join("\n\n"))
  end
end
